
# ---------------------------------------------------------
#    ____          __      ___     
#   |  _ \         \ \    / (_)    
#   | |_) | __ _ _ _\ \  / / _ ____
#   |  _ < / _` | '__\ \/ / | |_  /
#   | |_) | (_| | |   \  /  | |/ / 
#   |____/ \__,_|_|    \/   |_/___|  Simplex class
# ---------------------------------------------------------                            
# SIMaP / Jean-Luc Parouty 2022    

import numpy as np
from math import sqrt, cos, sin, acos

from barviz.SimplexBuilder import SimplexBuilder
from barviz.Scrawler       import Scrawler
from barviz.Attributes     import Attributes
from barviz.Collection     import Collection


class Simplex(Collection):

    version = 1.01
    type    = 'Simplex'

    _attributes_default= { 
        'markers_colormap'  : 'Turbo',
        'markers_opacity'   : 1,
        'markers_size'      : 8,
        'markers_visible'   : True,
        'markers_line_color': 'darkgray',
        'markers_line_width': 1,
        'lines_colormap'    : 'Turbo',
        'lines_opacity'     : 1,
        'lines_width'       : 2,
        'lines_visible'     : True,
        'text_opacity'      : 1,
        'text_size'         : 10,
        'text_visible'      : True,
        'plotly_template'   : 'plotly_white',
        'width'             : 500,
        'height'            : 500,
        'camera_eye'        : dict(x=.8, y=.8, z=.8),
        'save_scale'        : 6,
        'renderer'          : 'jupyterlab' 
    }

    def __init__( self,  points=[], name='unknown', colors=None, labels=None, attrs={} ):
        super(Simplex, self).__init__(points,name,colors,labels,attrs)
        self.version  = Simplex.version
        self._attrs   = Attributes(attrs, Simplex._attributes_default)
        self.scrawler = Scrawler(self)

    @classmethod
    def build(cls, nbp=5):
        spb = SimplexBuilder()
        spb.add_points(nbp)
        spb.iterate()
        return spb.get_simplex()


    def get_skeleton(self):
        n=self.nbp
        # ---- Vertices in barycentric world
        points=np.zeros( (n,n) )
        for i in range(n):
            points[i,i]=1

        # ---- Edges
        edges,colors,labels=[],[],[]
        for i in range(0,n):
            pi = points[i]
            for j in range(i+1,n):
                pj = points[j]
                edges.extend( (pi,pj,[None]*n) )
                colors.extend( (i,j,0) )
                labels.extend( (self.labels[i], self.labels[j], '') )

        return Collection(edges,'Skeleton',colors,labels,attrs=self.attrs.to_dict())


    def plot(self, *args, **kwargs):
        self.scrawler.plot(*args, **kwargs)

    def plot_save(self, *save_as):
        self.scrawler.plot_save(save_as)

    def resize(self, size=2):
        self.points = self.points * size / self.diagomax
